
const id     = "{22832dda-1de6-4443-9ab5-d34214b03347}";
const _debug = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
const _error = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
const _prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
const _cis   = Components.classes["@mozilla.org/intl/converter-input-stream;1"].createInstance(Components.interfaces.nsIConverterInputStream);

const _preference_root = "extensions.flashtracer.";
const _styles_root     = "styles.";
const _patterns_root   = "patterns.";

const settingsURL = "chrome://trace/content/settings.xul";
const helpURL     = "chrome://trace/content/help.xul";
const newStyleURL = "chrome://trace/content/dialogs/new_style.xul";
const aboutURL    = "chrome://trace/content/dialogs/about.xul";
const traceURL    = "chrome://trace/content/trace.xul";

// log message to the error console
function log( msg ) {
    if( _debug ){
        _debug.logStringMessage( msg );
    }
}

function error( msg ) {
    if( _error ) {
        _error.logStringMessage( msg );
    }
    singleMessage( msg );
}


function getFirefoxWindow() {
    var mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
	.getInterface(Components.interfaces.nsIWebNavigation)
	.QueryInterface(Components.interfaces.nsIDocShellTreeItem)
	.rootTreeItem
	.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
	.getInterface(Components.interfaces.nsIDOMWindow) 
    return mainWindow;
}

/**
 * Return this extension folder
 * @returns String
 */
function getPluginFolder(){
	netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
	var extDir = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager).getInstallLocation( id ).getItemLocation( id );
	return extDir;
}

// get the running operating system
function getOS(){
    var agt = navigator.userAgent.toLowerCase();
    if(agt.indexOf('win') != -1) {
	return "win";
    } else if(agt.indexOf("macintosh") != -1){
        return "mac";
    } else {
        return "linux";
    }
}


// retrieve the location of the mm.cfg file
// based on the current operating system
function getMMFile() {
    netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    var mmfile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    var file   = getMMDirectory();
    mmfile.initWithPath( file.path );
    mmfile.append("mm.cfg");
    return mmfile;
}

// return the mm.cfg directory
function getMMDirectory() {
    netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    var file;
    var path;
    file   = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("Home", Components.interfaces.nsIFile);
    return file;
}

// attempt to create the mm.cfg file
function saveMMFile() {
    netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    var str = "TraceOutPutFileName=" + (getFlashLogFile() == null ? "" : getFlashLogFile()) + "\nErrorReportingEnable=" + displayError + "\nTraceOutputFileEnable=1\nMaxWarnings=" + maxWarnings;
    var file = getMMFile();
    var is;    
    try {
	    if(!file.exists())
	    {
		    file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0777);
	    }
	    is = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
	    try
	    {
	    	is.init( file, 0x04|0x08|0x20, 0777, 0);
		} catch( error )
		{
			return error;
		}
	    is.write(str, str.length);
	    is.close();
	    return true;
    } catch(err) {
	    return err;
    }
}

// clear the contents of the flashlog.txt file
function flushFile( fname ) {
	netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
	try {
	    file.initWithPath( fname );
	    var is = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance( Components.interfaces.nsIFileOutputStream );
	    is.init( file, 0x04|0x08|0x20, 755, 0);
	    is.write("", 0);
	    is.close();
	    return true;
	} catch (e) {
	    return e;
	}
}

// ----------------------
// preferences
// ----------------------


// get the preferences branch
function getPreferences( branch ) {
    return _prefs.getBranch(_preference_root + (branch == null ? "" : branch + "."));
}

function setCharPref(p, name, value){
    p.setCharPref(name, value);
}

function getCharPref(p, name, defaultValue){
    try
    {
        return p.getCharPref(name);
    } catch(err) {
	return defaultValue;
    }
}

function setIntPref(p, name, value) {
    p.setIntPref(name, value);
}

function getIntPref(p, name, defaultValue) {
    try {
	return p.getIntPref(name);
    } catch(err) {
	return defaultValue;
    }
}

function getBoolPref(p, name, defaultValue) {
    try {
	return p.getBoolPref(name);
    } catch(e) {
	return defaultValue;
    }
}


function setBoolPref(p, name, value) {
    try {
	p.setBoolPref(name, value);
    } catch(e) {
    }
}



// -------------------------
// String functions
// -------------------------
function leftTrim( value ) {
    var str = value;
    while (str.substring(0,1) == ' ') {
	str = str.substring(1, str.length);
    }
    return str;
}

function rightTrim( value ) {
    var str = value;
    while (str.substring(str.length-1, str.length) == ' ') {
	str = str.substring(0, str.length-1);
    }
    return str;
}


function trim( value ) {
    return rightTrim(leftTrim(value));
}
