
//
// FlashTracer 2.3.1
// Alessandro Crugnola
// alessandro@sephiroth.it - http://www.sephiroth.it
//

var default_timeout = 50;
var timeout      = 50;  // main timeout for file reading
var is_playing   = false;
var modification = null;
var filesize     = -1;
var tid          = -1;  // main timeout ID
var fid          = -1;  // filter timeout

var os         = "win";
var mmfile     = null;
var mmdir      = null;
var mm_exist   = false;

var log_file      = null;  // flashlog.txt file
var displayError  = 1;
var maxWarnings   = 100;
var maxLines      = 0;

var preferences   = null;
var font_family   = "";
var font_size     = 10;
var trace_charset = "ISO-8859-1";
var filter_text   = "";
var old_filter_text = "";
var display_only_rules = 0;
var lock_position = false;
var pattern_array = []; // used for rule style filters
var text_values   = []; // all the text values here...

var display_inline = false;


// main entry point
function onCreationComplete()
{
    setIsPlaying( is_playing );
    
    preferences         = getPreferences();
    os                  = getOS();
    maxWarnings         = getIntPref(preferences, "common.MaxWarnings", 100);
    maxLines            = getIntPref(preferences, "common.MaxLines", 100);
    displayError        = getIntPref(preferences, "common.DisplayErrors", 1);
    font_family         = getCharPref(preferences, "common.font-family", "monospace");
    font_size           = getIntPref(preferences,  "common.font-size", 12);
    display_only_rules  = getIntPref( preferences, "common.display-only-rules", 0);
    lock_position       = getBoolPref( preferences, "common.scroll-lock", false );
    filter_text         = getCharPref( preferences, "common.filter-text", "");
    display_inline      = getBoolPref(preferences, "common.display-inline", 1);
    trace_charset       = getCharPref( preferences, "common.charset", "ISO-8859-1" );
    
    setLockPosition( lock_position );
    setDisplayInline( display_inline );
    
    mmdir       = getMMDirectory();
    mmfile      = getMMFile();
    
    log( "Welcome to flashtracer 2.3.1!" );
    log( 'mmdir: '  + mmdir.path );
    log( 'mmfile: ' + mmfile.path );

    document.getElementById('filter').value = filter_text;
    document.getElementById('displayOnlyRules').setAttribute("checked", display_only_rules == 1 ? true : false);
    saveCSS();
    
    singleMessage( 'using: ' + mmfile.path );
    initMMFile( false );
    initPatterns();

    prefObserver.register( preferences.root );

	// close the flash tracer popup window if already opened
	var ff = getFirefoxWindow();
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
	var enumerator = wm.getEnumerator(null);
	while(enumerator.hasMoreElements()) {
		var win = enumerator.getNext();
		if(win.name == "tracerPopupWindow" && this.name != "tracerPopupWindow"){
			win.close();
		}
	}
    
    document.getElementById('isPopupWindow').setAttribute('disabled', this.name == "tracerPopupWindow");
    log(log_file);
}

// user input on the main filter input
function onFilterTextChanged() {
    var value = document.getElementById('filter').value;
    
    try {
        "test".search( value );
        document.getElementById('filter').inputField.style.backgroundColor = '#FFFFFF';
        document.getElementById('filter').inputField.style.color = '#000000';
    } catch( err ) {
        document.getElementById('filter').inputField.style.backgroundColor = '#FF6666';
        document.getElementById('filter').inputField.style.color = '#FFFFFF';
        return;
    }
    
    setCharPref(preferences, "common.filter-text", value );
    clearTimeout( fid );
    
    if( is_playing ) {
    } else {
        fid = setTimeout("onFilterTextTimeout('" + value + "')", 500 );
    }
}

function onFilterTextTimeout( value )
{
    displayValues( );
}

// clear both the flashlog.txt file and the output
function onClearFlashLog()
{
    try {
        flushFile( log_file );
    } catch(err) {
        singleMessage(err);
    }
    modification = null;
    filesize = -1;
    clearOutput();
}


// clear the output html element
function clearOutput( ) {
    getContentList().innerHTML = "Welcome to flashtracer!<br />by Alessandro Crugnola<br />http://www.sephiroth.it";
}

/**
 * toggle popup
 *
 */
function onTogglePopup()
{
    var ff = getFirefoxWindow();
    if(this.name != "tracerPopupWindow") {
	tracerPopupWindow = window.open(traceURL + "?ispopup=true", "tracerPopupWindow", "chrome,resizable=yes,height=400,width=300");
	ff.toggleSidebar('viewTraceSearch');
    } else {
	ff.toggleSidebar('viewTraceSearch');
	this.close();
    }
}

function getFlashLogFileExists() {
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    try
    {
    	file.initWithPath( getFlashLogFile() );
	} catch( error )
	{
		return false;
	}
    return file.exists();
}

// attempt to create the mm.cfg file
// and initialize the log_file variable
// start playing if everything is ok
function initMMFile( force )
{
    var mm_exists = true;
    if( !getFlashLogFileExists() )
    {
	singleMessage("flashlog.txt file does't exists. Open settings dialog first.");
	return false;
    }
    
    if( (!mmfile.exists()) || mmfile.fileSize == 0 || force )
    {
        if( mmdir.isWritable() )
        {
            var result = saveMMFile();
            if( result != true )
            {
                error( "Cannot create the mm.cfg file in " + mmfile.path );
                error( result );
		mm_exists = false;
		setMMExist( false );
            } else {
                mm_exists = true;
		setMMExist( true );
            }
        } else {
            error(mmdir.path + " is not writeable, please check permissions");
            setMMExist( false );
	    mm_exists = false;
        }
    }
    
    if( mm_exists )
    {
	setMMExist( true );
	log_file = getFlashLogFile();
	if( log_file != null )
	{
	    play();
	} else {
	    singleMessage('flashlog.txt has not been defined in preferences yet');
        }
    }
}

// initialize the patterns array
function initPatterns()
{
    pattern_array = [];
    var label;
    var _tmp = [];
    var pEnumerator = preferences.getChildList(_patterns_root, _tmp);
    for( var a = 0; a < pEnumerator.length; a++) {
        label = pEnumerator[a].substring( _patterns_root.length );
        pattern_array.push({ label: label, pattern: new RegExp( getCharPref(preferences, _patterns_root + label, ""), "i")});
    }
}

// Set the current playing mode
function setIsPlaying( value )
{
    var element = document.getElementById('isPlaying');
    if( !value )
    {
        element.setAttribute("label", "Play");
        element.setAttribute("style", "list-style-image: url('chrome://trace/skin/play.png');");
    } else {
        element.setAttribute("label", "Pause");
        element.setAttribute("style", "list-style-image: url('chrome://trace/skin/pause.png');");
    }
}


function setLockPosition( value )
{
    var element = document.getElementById('isLockPosition');
    element.setAttribute("checked", value);
}

// click on Lock checkbox
function onLockPositionChange( )
{
	setBoolPref( preferences, "common.scroll-lock", document.getElementById("lock_position").checked );
}

// check on the toolbar checkbox 'only rules'
function onDisplayOnlyRules( )
{
    setIntPref(preferences, "common.display-only-rules", document.getElementById('display-only-rules').checked ? 1 : 0);
}

// set the checked value of Inline checkbox
function setDisplayInline( value )
{
    var element = document.getElementById('displayInline');
    element.setAttribute("checked", value);
}

// Refresh the CSS and the displayed text
function onDisplayInline()
{
	setBoolPref( preferences, "common.display-inline", document.getElementById("display-inline").checked ? 1 : 0 );
	display_inline = document.getElementById("display-inline").checked;
	saveCSS();
	displayValues();
}

// clear the filter textbox
function onClearFilter()
{
	document.getElementById('filter').value = "";
	document.getElementById('cmd_filter').doCommand();
}

function onPlayPause()
{
    if( is_playing )
    {
        pause();
    } else {
        play();
    }
}

function play()
{
    is_playing = true;
    setIsPlaying( is_playing );
    tid = setTimeout( "readfile()", timeout );
}

function pause()
{
    is_playing = false;
    setIsPlaying( is_playing );
    clearTimeout( tid );
}

// read the flashlog file
function readfile()
{
    netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(log_file);
    
    if( !file.exists() ) {
        singleMessage("flashlog.txt file does not exists, please check your settings");
        pause();
        return;
    }
    
    var cis = Components.classes["@mozilla.org/intl/converter-input-stream;1"].createInstance( Components.interfaces.nsIConverterInputStream );

    var output = "";
    var hasmore;
    var line = {};
    var modified = false;
	
    try
    {
        if( modification != file.lastModifiedTime || filesize != file.fileSize || old_filter_text != filter_text ) {		
            modified = true;
            text_values = [];
            modification = file.lastModifiedTime;
            filesize     = file.fileSize;
            old_filter_text = filter_text;
            var fis = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance( Components.interfaces.nsIFileInputStream );
            fis.init( file, 0x01, 00004, null );
            cis.init( fis, trace_charset, 1024, Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER );
            if( cis instanceof Components.interfaces.nsIUnicharLineInputStream )
            {
                do
                {
                    hasmore = cis.readLine( line );
                    text_values.push( line.value );
                } while ( hasmore );
                cis.close( );
            }
            fis.close( );
            displayValues( );
        }
        
    } catch(e)
    {
        error( e );
        pause( );
    }
    
    if( is_playing )
    {
        if( modified )
        {
            timeout = default_timeout;
        } else {
            timeout = 1000;
        }
        play();
    }
}


// enable/disable buttons depending on existence of the mm.cfg file
function setMMExist( value ) {
    var element = document.getElementById('isPlaying');
    element.setAttribute("disabled", value == true ? "false" : "true" );
}

// return the registered flashlog.txt file
function getFlashLogFile() {
    return getCharPref(preferences, "common.flashlog", null);
}


// display a single message on the output panel
function singleMessage(value) {
    getContentList().innerHTML += value + "<br />";
}

function match_pattern( line )
{
    var e;
    for( var a = 0; a < pattern_array.length; a++) {
	try {
	    if( line.match(pattern_array[a]['pattern']) != null ) {
		return pattern_array[a]['label'];
	    }
	} catch(e) {
	}
    }
    return "";
}

// trace messages using rules
function displayValues( ) {
    var list_box = getContentList();
    var ddc      = getContentDocument();
    var currentScroll  = ddc.documentElement.scrollTop;
    var maxScroll      = ddc.documentElement.scrollHeight;
    var item;
    var text      = "";
    var output    = new Array( );
    var div_class = "";
    
    for(var a = 0; a < text_values.length; a++)
    {
        text      = text_values[a];
        div_class = match_pattern( text );
        text      = text.replace(/</g, "&lt;");
        text      = text.replace(/>/g, "&gt;");
        //text = text.replace(/\s\t/g, "&nbsp;");
        if(display_inline) {
            item = "<div style='white-space: pre; padding: 0; margin: 0; display: inline;' class='" + div_class + "'>" + text + "</div>\n";
        } else {
            item = "<div style='padding: 0; margin: 0;' class='" + div_class + "'>" + text + "</div>\n";
        }
        
        if( display_only_rules == 1 && div_class == "" && pattern_array.length > 0) {
            continue;
        }

        // using filter
        if( filter_text != null && filter_text != "" ) {
            if( text_values[a].search( filter_text ) < 0 ) {
                continue;
            }
        }
        //output += item;
        output.push( item );
    }
    
    if( maxLines > 0 )
    {
        var array_start = Math.max( output.length - maxLines, 0);
        output = output.slice( array_start );
    }
    
    list_box.innerHTML = output.join("");
    
    if( lock_position ) {
        if( currentScroll != maxScroll ) {
            ddc.documentElement.scrollTop = currentScroll;
        } else {
            ddc.documentElement.scrollTop = ddc.documentElement.scrollHeight;
        }
    } else {
        ddc.documentElement.scrollTop = ddc.documentElement.scrollHeight;
    }
}

// get the trace.xul iframe content document
function getContentDocument()
{
    return document.getElementById("content-body").contentDocument;
}

// get the main output element inside the iframe document
function getContentList() {
    return getContentDocument().getElementById("contentlist");
}

function getContentCSS() {
    return getContentDocument().getElementById('style-content');
}

// display the settings dialog
function openSettingDialog() {
    var params = {inn: {}, out: null};
    params.inn.log_file       = log_file == null ? "" : log_file;
    params.inn.display_errors = displayError;
    params.inn.max_warnings   = maxWarnings;
    params.inn.max_lines      = maxLines;
    params.inn.charset        = trace_charset;
    params.inn.font_family    = font_family;
    params.inn.font_size      = font_size;
    
    var dlg = window.openDialog(settingsURL, "", "chrome,modal,close", params).focus();
    if( params.out ) {
        setCharPref(preferences, "common.flashlog",      params.out.log_file );
        setIntPref(preferences,  "common.MaxWarnings",   params.out.max_warnings );
        setIntPref(preferences,  "common.MaxLines",      params.out.max_lines );
        setIntPref(preferences,  "common.DisplayErrors", params.out.display_errors );

        setCharPref(preferences, "common.charset",       params.out.charset );
        setCharPref(preferences, "common.font-family",   params.out.font_family)
        setIntPref(preferences,  "common.font-size",     params.out.font_size);

        initMMFile( true );
        saveCSS();
    }
}

/**
 * Save the css file for display contents
 *
 */
function saveCSS()
{
    getContentCSS().innerHTML = getFormattedStyles();
}

/**
 * Return the formatted css style defined
 * from the list of styles
 */
function getFormattedStyles()
{
    var ret = "";
    var label;
	
    p_array = [];
    pEnumerator = preferences.getChildList( _patterns_root, p_array);
    for(var a = 0; a < pEnumerator.length; a++) {
	label = pEnumerator[a].substring( _patterns_root.length );
	var stylePattern = getCharPref(preferences, _patterns_root + label, "");
	var style = getCharPref(preferences, _styles_root + label, "");
	
	ret += "." + label + " {\r\n";
	ret += "    " + style + ";\r\n";
	ret += "}\r\n\r\n";
    }
    
    ret += "code {\r\n";
    ret += "    font-family: " + font_family + ";\r\n";
    ret += "    font-size: " + font_size + "px;\r\n";
	if(display_inline)
	{
		ret += "    white-space: pre;\r\n";
	}
	else
	{
		ret += "    white-space: normal;\r\n";
	}    
    ret += "}\r\n\r\n";
    return ret;
}


// preferences observer
prefObserver =
{
	register: function( rootNode )
	{
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		this._branch = prefService.getBranch( rootNode );
		this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
		this._branch.addObserver("", this, false);
	},
    
	unregister: function()
	{
		if(!this._branch) return;
		this._branch.removeObserver("", this);
	},
    
	observe: function(aSubject, aTopic, aData)
    {
        if(aTopic != "nsPref:changed") return;
        switch(aData)
        {
            case "common.MaxWarnings":
                maxWarnings = getIntPref( preferences, "common.MaxWarnings", 100);
                break;
            case "common.MaxLines":
                maxLines = getIntPref( preferences, "common.MaxLines", 0 );
                break;
            case "common.DisplayErrors":
                displayError = getIntPref( preferences, "common.DisplayErrors", 1);
                break;
            case "common.charset":
                trace_charset = getCharPref( preferences, "common.charset", "ISO-8859-1" );
                modification = null;
                filesize = -1;
                break;
            case "common.font-family":
                font_family = getCharPref( preferences, "common.font-family", "monospace");
                modification = null;
                filesize = -1;
                break;
            case "common.font-size":
                font_size = getIntPref( preferences, "common.font-size", 10);
                modification = null;
                filesize = -1;
                break;
            case "common.display-only-rules":
                display_only_rules = getIntPref( preferences, "common.display-only-rules", 0);
                document.getElementById('displayOnlyRules').checked = display_only_rules;
                modification = null;
                filesize = -1;
                displayValues( );
                break;
			case "common.scroll-lock":
				lock_position = getBoolPref( preferences, "common.scroll-lock", false );
                setLockPosition(lock_position);
				break;
			case "common.inline":
				display_inline = getBoolPref( preferences, "common.display-inline", 1 );
	            break;
            case "common.filter-text":
                filter_text = getCharPref( preferences, aData, "");
                break;
            default:
                initPatterns();
                break;
        }
    }
}
